#include "bme2wav.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>

bool main_callback( int percentage );
bool wav_file_empty_callback( char *empty_filename );
bool wav_file_empty_ignore = false;

void usage() {
  fprintf( stderr, "Usage: bme2wav [-v volume(%%)] [-o output] [-d] [-i] [-r] <input>\n" );
  exit( 0 );
}

int main( int argc, char *argv[] ) {
  bool status;
  int volume = 50;
  char *input_filename = NULL;
  char *output_filename = NULL;
  char *p;
  bool debug = false;
  bool resounding = false;
  
  for ( int i = 1; i < argc; ++i ) {
    if ( argv[i][0] == '-' ) {
      switch( argv[i][1] ) {
      case 'v':
        ++i;
        if ( i >= argc ) { usage(); }
        volume = strtol( argv[i], &p, 10 );
        if ( argv[i] == p ) { usage(); }
        break;

      case 'o':
        ++i;
        if ( i >= argc ) { usage(); }
        output_filename = argv[i];
        break;

      case 'd':
        debug = true;
        break;

      case 'i':
        wav_file_empty_ignore = true;
        break;

      case 'r':
        resounding = true;
        break;

      default:
        usage();
      }
    }
    else if ( input_filename == NULL ) {
      input_filename = argv[i];
    }
    else { usage(); }
  }
  
  if ( input_filename == NULL ) { usage(); }
  if ( output_filename == NULL ) {
    output_filename = new char[MAX_PATH];
    strcpy( output_filename, input_filename );
    TtPath::edit_path( output_filename, true, true, true, false );
    strcat( output_filename, ".wav" );
  }

  printf( "INPUT BMS FILE  : %s\n", input_filename );
  printf( "OUTPUT WAV FILE : %s\n", output_filename );
  printf( "VOLUME          : %d%%\n", volume );
  
  BmsConverter *converter = new BmsConverter();
  converter->set_debug( debug );
  converter->set_resounding( resounding );
  converter->set_bms_path( input_filename );
  BmsBms *bms = converter->open_bms();
  if ( bms == NULL_BMS ) {
    fprintf( stderr, "\nBMSt@C͎̉s %s\n", input_filename );
    fprintf( stderr, "line %d : %s\n",  BmsLexer::get_last_error_line_num(), BmsLexer::get_last_error_str() );
    exit( -1 );
  }
  FILE *_tmp = fopen( output_filename, "wb" );
  if ( _tmp == NULL ) {
    fprintf( stderr, "\no̓t@Cɏ߂܂ %s\n", output_filename );
    exit( -1 );
  }
  fclose( _tmp );
  converter->set_master_volume( volume );
  converter->set_callback( main_callback );
  converter->set_wav_file_empty_callback( wav_file_empty_callback );
  TtWav *out_wav = converter->convert();
  if ( out_wav == NULL ) {
    fprintf( stderr, "\nWAVւ̕ϊs\n" );
    exit( -1 );
  }

  if ( out_wav->write_file( output_filename ) ) {
    fprintf( stderr, "\nWAVt@C֏݂܂ %s\n", output_filename );
  }
  else {
    fprintf( stderr, "\nWAVt@Cւ݂̏s %s\n", output_filename );
    exit( -1 );
  }
  return 0;
}


bool main_callback( int percentage ) {
  if ( percentage == -1 ) {
    printf( "WAVϊ ......\n" );
  }
  else if ( percentage == 0 ) {
    printf( "WAVǂݍݒ\n" );
    printf( "0----------------------------------------------100\n" );
  }
  else if ( percentage < 100 && percentage % 2 == 0 ) {
    printf( "*" );
  }
  else if ( percentage == 100 ) {
    printf( "*\n" );
    printf( "WAVJn\n" );
    printf( "0----------------------------------------------100\n" );
  }
  else if ( percentage < 200 && percentage % 2 == 0 ) {
    printf( "*" );
  }
  else if ( percentage == 200 ) {
    printf( "*\n" );
  }
  return true;
}

bool wav_file_empty_callback( char *empty_filename ) {
  return wav_file_empty_ignore;
}
