// WIN32  EBhERg[pNXQ
#ifndef _TTWINDOW_H_
#define _TTWINDOW_H_
#define OEMRESOURCE

#include <windows.h>
#include <commctrl.h>

// -- TtWindow -----------------------------------------------------------
class TtWindow {

  // -- static -----------------------------------------------------------
public:
  static const int MSG_DEFAULT;

  // -- message dispatch system ------------------------------------------
public:
  static LRESULT CALLBACK _ttl_wnd_proc( HWND h_wnd, UINT msg, WPARAM w_param, LPARAM l_param );
  static TtWindow *find_window( HWND h_wnd );
  static void init_common_controls();

  // -- override at sub class --------------------------------------------
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();
  virtual bool created();

public:
  virtual LRESULT wnd_proc( UINT msg, WPARAM w_param, LPARAM l_param );
  virtual LRESULT wm_hscroll_recved( int code, int position, HWND h_wnd );
  virtual LRESULT wm_vscroll_recved( int code, int position, HWND h_wnd );

  // -- instance ---------------------------------------------------------
protected:
  TtWindow *parent;
  HWND h_wnd;
  int id;
  HINSTANCE h_instance;
  WNDPROC wnd_proc_super;

  int override_wnd_proc();

public:
  TtWindow();
  bool create( TtWindow *parent );
  bool create( TtWindow *parent, int id );
  bool create( TtWindow *parent, int id, HINSTANCE h_instance );
  bool create( TtWindow *parent, int id, HINSTANCE h_instance, LPVOID lp_param );
  bool create_not_child( TtWindow *parent, int id, HINSTANCE h_instance, LPVOID lp_param );

  bool _create( TtWindow *parent, int id, HINSTANCE h_instance, LPVOID lp_param, bool child );

  int close();
  ~TtWindow();
  
  HWND get_h_wnd();
  int get_id();
  HINSTANCE get_h_instance();

  TtWindow *get_parent();
  RECT get_rect();
  int get_pos_x();
  int get_pos_y();
  int get_width();
  int get_height();

  int show();
  int hide();
  int minimize();
  int maximize();
  int restore();
  int set_show_state( int state );

  int set_position( int x, int y );
  int set_size( int width, int height );
  int set_pos_size( int x, int y, int width, int height );
  int set_rect( RECT rect );

  bool set_bottom();
  bool set_top();
  bool set_under( HWND wnd );

  int set_font( HFONT h_font );

  int set_forground();

  int get_text( char *buf, int buf_size );
  int set_text( const char *str );

  int messageloop();
};

// -- TtStatic -----------------------------------------------------------
class TtStatic : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();
};

// -- TtButton -----------------------------------------------------------
class TtButton : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();
};

// -- TtCheckBox ----------------------------------------------------------
class TtCheckBox : public TtButton {
protected:
  virtual DWORD  get_style();

public:
  bool get_check();
  int set_check( bool checking );
};

// -- TtGroup ------------------------------------------------------------
class TtGroup : public TtButton {
protected:
  virtual DWORD  get_style();
};

// -- TtEdit -------------------------------------------------------------
class TtEdit : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();
};

// -- TtList -------------------------------------------------------------
class TtList : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

public:
  int get_count();
  int get_text_at( int index, char *buf );
  int get_current();
  int find( int start, const char *str );

  int push( const char *str );
  int insert_at( int index, const char *str );

  int delete_at( int index );
  void clear();

  int get_selected_count();
  bool is_select( int index );
  int set_select( int index );
  int unset_select( int index );

  int lock_draw();
  int unlock_draw();
};

// -- TtScroll -----------------------------------------------------------
class TtScroll : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

  SCROLLINFO info;

public:

  virtual int get_track_position();
  virtual void set_track_position( int position );

  virtual int get_min();
  virtual void set_min( int min );
  virtual int get_max();
  virtual void set_max( int max );
  virtual void set_range( int min, int max );

  virtual int get_page_size();
  virtual void set_page_size( int page_size );

protected:
  bool get_info();
  void set_info();

  LRESULT scrolling( int code, int position );
};

// -- TtScrollV ----------------------------------------------------------
class TtScrollV : public TtScroll {
protected:
  virtual DWORD  get_style();
  virtual LRESULT wm_vscroll_recved( int code, int position, HWND h_wnd );
};

// -- TtScrollH ----------------------------------------------------------
class TtScrollH : public TtScroll {
protected:
  virtual DWORD  get_style();
  virtual LRESULT wm_hscroll_recved( int code, int position, HWND h_wnd );
};

// -- TtTrack ------------------------------------------------------------
class TtTrack : public TtScroll {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

public:
  virtual int get_track_position();
  virtual void set_track_position( int position );

  virtual int get_min();
  virtual void set_min( int min );
  virtual int get_max();
  virtual void set_max( int max );
  virtual void set_range( int min, int max );

  virtual int get_page_size();
  virtual void set_page_size( int page_size );

  virtual int get_line_size();
  virtual void set_line_size( int line_size );

  void set_tic_frequency( int frequency );
  void clear_tic();

protected:
  virtual LRESULT wm_vscroll_recved( int code, int position, HWND h_wnd );
  virtual LRESULT wm_hscroll_recved( int code, int position, HWND h_wnd );

};

// -- TtTrackV -----------------------------------------------------------
class TtTrackV : public TtTrack {
protected:
  virtual DWORD  get_style();
};

// -- TtTrackH -----------------------------------------------------------
class TtTrackH : public TtTrack {
protected:
  virtual DWORD  get_style();
};

// -- TtTab --------------------------------------------------------------
class TtTab : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

public:
  int get_count();

  int insert_at( int index, const char *str );

  int delete_at( int index );
  int clear();

  int set_current( int index );
  int get_current();

  RECT get_view_rect();
};

// -- TtProgress ---------------------------------------------------------
class TtProgress : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

public:
  int get_max();
  int get_amount();

  int set_max( int max );
  int set_step( int step );

  int set_amount( int position );

  int step_up( int amount );
  int step_it();
};

// #define TreeView_GetItemRect(hwnd, hitem, prc, code)
// #define TreeView_GetIndent(hwnd)
// #define TreeView_SetIndent(hwnd, indent)
// #define TreeView_GetImageList(hwnd, iImage)
// #define TreeView_SetImageList(hwnd, himl, iImage)
// #define TreeView_Select(hwnd, hitem, code)
// #define TreeView_SelectDropTarget(hwnd, hitem)
// #define TreeView_SelectSetFirstVisible(hwnd, hitem)
// #define TreeView_GetEditControl(hwnd)
// #define TreeView_HitTest(hwnd, lpht)
// #define TreeView_CreateDragImage(hwnd, hitem)
// #define TreeView_EnsureVisible(hwnd, hitem)
// #define TreeView_EndEditLabelNow(hwnd, fCancel)
// #define TreeView_SetToolTips(hwnd,  hwndTT)
// #define TreeView_GetToolTips(hwnd)
// #define TreeView_GetISearchString(hwndTV, lpsz)
// #define TreeView_SetInsertMark(hwnd, hItem, fAfter)
// #define TreeView_SetUnicodeFormat(hwnd, fUnicode)
// #define TreeView_GetUnicodeFormat(hwnd)
// #define TreeView_SetItemHeight(hwnd,  iHeight)
// #define TreeView_GetItemHeight(hwnd)
// #define TreeView_SetBkColor(hwnd, clr)
// #define TreeView_SetTextColor(hwnd, clr)
// #define TreeView_GetBkColor(hwnd)
// #define TreeView_GetTextColor(hwnd)
// #define TreeView_SetScrollTime(hwnd, uTime)
// #define TreeView_GetScrollTime(hwnd)
// #define TreeView_SetInsertMarkColor(hwnd, clr)
// #define TreeView_GetInsertMarkColor(hwnd)
// #define TreeView_SetItemState(hwndTV, hti, data, _mask)
// #define TreeView_SetCheckState(hwndTV, hti, fCheck)
// #define TreeView_GetItemState(hwndTV, hti, mask)
// #define TreeView_GetCheckState(hwndTV, hti)
// #define TreeView_SetLineColor(hwnd, clr)
// #define TreeView_GetLineColor(hwnd)

// -- TtTreeView ---------------------------------------------------------
class TtTreeView : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

  TtTreeItem *root;
  int default_selected_image_index;
  
public:
  TtTreeView();
  ~TtTreeView();
  
  TtTreeItem *get_root();
  TtTreeItem *get_selected();
  TtTreeItem *get_drop_hilight();
  
  int get_count();
  int get_visible_count();

  void set_image_list( HIMAGELIST himl );
  void set_image_list_state( HIMAGELIST himl );
};

// -- TtTreeItem ---------------------------------------------------------
class TtTreeItem {
  friend TtTreeView;
  
  static const int TABLE_SIZE;
  
protected:
  TtTreeView *tree;
  TVINSERTSTRUCT *substance;
  TtTreeItem *parent;
  TtTreeItem *childs;
  TtTreeItem *next;
  int child_image_index;
  int child_selected_image_index;
  
  bool add_child( TtTreeItem *child );
  bool remove_child( TtTreeItem *child );
  TtTreeItem *find_child( HTREEITEM item );
  TtTreeItem *find_descendant( HTREEITEM item );

  TtTreeItem();

  bool do_get();
  bool do_set();

public:
  TtTreeItem( TtTreeView *tree );
  ~TtTreeItem();

  HTREEITEM get_handle();
  TtTreeItem *get_parent();
  TtTreeItem *get_first_child();
  TtTreeItem *get_next_brother();
  TtTreeItem *get_prev_brother();
  
  void insert( TtTreeItem *item );
  void insert_first( TtTreeItem *item );
  void insert_last( TtTreeItem *item );
  void insert_sorted( TtTreeItem *item );
  TtTreeItem *insert_text( char *str );
  void sort_childlen();
  void sort_childlen_recursive();

  bool unlink( TtTreeItem *item );
  bool remove( TtTreeItem *item );
  void remove_all();
  
  void set_text( char *str );
  void get_text( char *buf, int buf_size );
  void set_edit_mode();
  void set_select();

  void expand();
  void collapse();
  void toggle_expand();

  void set_image_index( int index );
  void set_selected_image_index( int index );
  void set_both_image_index( int index );
  
  int  get_child_image_index();
  void set_child_image_index( int index );
  int  get_child_selected_image_index();
  void set_child_selected_image_index( int index );
};

// -- TtApp --------------------------------------------------------------
class TtApp : public TtWindow {
private:
  static char *class_name;

public:
  static bool init( HINSTANCE h_instance );

protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();
  virtual bool created();

  virtual LRESULT wm_command_recved( int id, int code, HWND child );
  virtual LRESULT wm_notify_recved( int id, NMHDR *nmhdr );
  virtual LRESULT wm_size_recved( int flag, int width, int height );
  virtual LRESULT wm_destroy_recved();

public:
  virtual LRESULT wnd_proc( UINT msg, WPARAM w_param, LPARAM l_param );
  virtual LRESULT wm_hscroll_recved( int code, int position, HWND h_wnd );
  virtual LRESULT wm_vscroll_recved( int code, int position, HWND h_wnd );

};

#endif // _TTWINDOW_H_
